<?php

$BATIK_PATH = 'batik-rasterizer.jar';
$TEMP_PATH = 'temp';

ini_set('magic_quotes_gpc', 'off');

$sType = isset($_POST['t'])?$_POST['t']:'png';
$sSvg = isset($_POST['svg'])?$_POST['svg']:'';
$sFileName = (isset($_POST['fn']) && trim($_POST['fn']) != '')?$_POST['fn']:('chart-'.uniqid());

if (get_magic_quotes_gpc()) {
	$sSvg = stripslashes($sSvg);
}

$sTempName = md5(rand());

// extract just the <svg></svg> part
$iPos0 = strpos($sSvg, '<svg');
$iPos1 = strpos($sSvg, '</svg>', $iPos0);
$sSvg = substr($sSvg, $iPos0, $iPos1-$iPos0+6);

// inject header
$sSvg = '<?xml version="1.0" standalone="no"?><!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.1//EN" "http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd">' . $sSvg;

// prevent batik errors due to incompatible characters
$sSvg = iconv('UTF-8', 'UTF-8//TRANSLIT//IGNORE', $sSvg);
//$sSvg = utf8_encode($sSvg);

// save raw (for debug)
file_put_contents($TEMP_PATH . '/raw.svg', $sSvg);

// IE fixes

// fix ie not putting quotes around attribute values
$sSvg = preg_replace('/id=([^"]+?)\s/', 'id="$1" ', $sSvg);
$sSvg = preg_replace('/class=([^"]+?)\s/', 'class="$1" ', $sSvg);
$sSvg = preg_replace('/transform([^=])/', '$1', $sSvg);
$sSvg = preg_replace('/width=([^"]+?)\s/', 'width="$1" ', $sSvg);
$sSvg = preg_replace('/height=([^"]+?)\s/', 'height="$1" ', $sSvg);
$sSvg = preg_replace('/<IMG(.*?)src=(.*?)>/', '<image$1xlink:href=$2></image>', $sSvg);
$sSvg = preg_replace('/<image(.*?) href=(.*?)><\/image>/', '<image$1 xlink:href=$2></image>', $sSvg);
$sSvg = preg_replace('/<image(.*?) href=(.*?)\/>/', '<image$1 xlink:href=$2></image>', $sSvg);

// fix <:circle and <:path tags generated by node cloning
$sSvg = str_replace(array('<:circle', '</:circle>', '<:path', '</:path>'), array('<circle', '</circle>', '<path', '</path>'), $sSvg);

// ignore non-reachable images
function img($matches) {
	if (!@fopen($matches[2], 'r')) {
		return '';
	}
	return $matches[0];
}
$sSvg = preg_replace_callback('/<image(.*?)xlink:href="(.+?)"(.*?)><\/image>/', 'img', $sSvg);

// fix empty textContent attributes
if (strpos($sSvg, 'textContent') !== false) {
	$sSvg = preg_replace('/textContent([^=])/', '$1', $sSvg);
	// move textContent attributes as tspan content
	$sSvg = preg_replace('/<tspan (.*?)textContent="(.*?)"(.*?)><\/tspan>/', '<tspan $1 $3>$2</tspan>', $sSvg);
}

// fix uppercase style causing render issues
function tlc($matches) {
	$aStyle = explode(';', $matches[1]);
	for ($i=0;$i<count($aStyle);$i++) {
		$aPV = explode(':', $aStyle[$i]);
		if (count($aPV) == 2) {
			$aStyle[$i] = strtolower($aPV[0]) . ':' . $aPV[1];
		}
	}
	return 'style="' . join(';', $aStyle) . '"';
}
$sSvg = preg_replace_callback('/style="(.+?)"/', 'tlc', $sSvg);

// force letter-spacing:-0.75 to fix font differences
// $sSvg = preg_replace('/<tspan (.*?)style="(.*?)"(.*?)>/', '<tspan $1style="letter-spacing:-0.75;$2"$3>', $sSvg);

// save last (for debug)
// file_put_contents($TEMP_PATH . '/last.svg', $sSvg);

// generate

$sContentType = '';
if ($sType == 'png') {
	$sContentType = 'image/png';
	$sExt = 'png';
} else if ($sType == 'jpeg') {
	$sContentType = 'image/jpeg';
	$sExt = 'jpg';
} else if ($sType == 'pdf') {
	$sContentType = 'application/pdf';
	$sExt = 'pdf';
}

$sConvertedFile = $TEMP_PATH . '/' . $sTempName . '.' . $sExt;

try {
	if ($sContentType == '') {
		throw new Exception('', 1000);
	}
	if (!file_put_contents($TEMP_PATH . '/' . $sTempName. '.svg', $sSvg)) {
		throw new Exception('', 1001);
	}

	$sOutput = shell_exec('java -jar ' . $BATIK_PATH . ' -m ' . $sContentType . ' -q 0.9 -d ' .  $sConvertedFile . ' ' . $TEMP_PATH . '/' . $sTempName . '.svg');
	// file_put_contents($TEMP_PATH . '/log.txt', $sOutput, FILE_APPEND);

	if (!is_file($sConvertedFile) || filesize($sConvertedFile) < 1) {
		throw new Exception('', 1002);
	} else {
		if (isset($_POST['base64'])) {
			echo 'data:' . $sContentType . ';base64,' . base64_encode(file_get_contents($sConvertedFile));
		} else {
			header('Content-Disposition: attachment; filename=' . $sFileName . '.' . $sExt);
			header('Content-Length: ' . filesize($sConvertedFile));
			header('Content-Type: ' . $sContentType);
			echo file_get_contents($sConvertedFile);
		}
	}

	@unlink($TEMP_PATH . '/' . $sTempName . '.svg');
	@unlink($sConvertedFile);
} catch (Exception $ex) {
	if (isset($_POST['base64'])) {
		echo 'data:image/png;base64,' . base64_encode(file_get_contents('error.png'));
	} else {
		header('Content-Length: ' . filesize('error.png'));
		header('Content-Type: image/png');
		echo file_get_contents('error.png');
	}
}

?>
